#!/usr/bin/env python

"""
CourseBuilder 

Coursebuilder is a preprocessor tool to generate curricula with pandoc from
structured representations. Data scheme and values are kept in YAML files 
in order to version content with Git.

"""

from argparse import ArgumentParser
import itertools
import yaml
import os
import textwrap

config_file = 'modulhandbuch.yaml'
line_length = 80
column_ratio= 0.28

def build_curriculum(input_path,lang='de',pagebreak=False,title=False):

    # open the config file
    file_path = os.path.realpath(__file__)
    config_path = os.path.join(os.path.dirname(file_path),config_file)
    transforms = None # for translation

    # load transforms
    with open(config_path,'r') as cf:
        transforms = yaml.load(cf,Loader=Loader)

    # open meta.yaml in the directory
    with open(os.path.join(input_path),'r') as fp:
    
        # get configuration data
        desc = yaml.load(fp,Loader=Loader)

        # collect transformations
        ti = []

        # fix for now
        for k,v in desc[lang].items():
            v = v if v else ""
            ti.append((transforms[lang][k],str(v)))  

        # to limit
        h_len = int(line_length * column_ratio)
        d_len = line_length-h_len

        if title:
            print('#',desc[lang]['name'],'\n')


        print(''.join(['+',"".ljust(h_len,'-'),'+',"".ljust(d_len,'-'),'+']))

        headline = False

        for k,v in ti:

            h =  textwrap.wrap(k, h_len, break_long_words=False)
            wrapper = textwrap.TextWrapper(d_len)
            t = [wrapper.wrap(i) for i in v.split('\n') if i != '']
            t = list(itertools.chain.from_iterable(t))
                  
            # get rows
            rows = list(itertools.zip_longest(h,t,fillvalue=""))

            # expand rows
            for r in rows:
                print(''.join(['|',r[0].ljust(h_len,' '),'|',r[1].ljust(d_len,' '),'|']))

            if headline:
                print(''.join(['+',"".ljust(h_len,'-'),'+',"".ljust(d_len,'-'),'+']))
            else:
                print(''.join(['+',"".ljust(h_len,'='),'+',"".ljust(d_len,'='),'+']))
                headline = True

        # to control pagebreaks for pandoc
        if pagebreak:
            print('\n\\newpage')

# create a meta file
def create_meta():
    
    # open the config file
    file_path = os.path.realpath(__file__)
    config_path = os.path.join(os.path.dirname(file_path),config_file)
    transforms = None # for translation

    # load transforms
    with open(config_path,'r') as cf:
        while line := cf.readline():
            print(line.rstrip())


class CourseBuilder:

    def __init__(self) -> None:
        self.__schema = None
    
    def set_schema(self,schema = None):
        self.__schema = schema
        # print(self.__schema)

    def process(self,meta = None,lang='en'):
        fields = ['name']

        for field in fields:
            print(self.__schema[field]['label'][lang],':',meta[field][lang])

    def process_int(self,lang='de'):        
        pass

    def process_enum(self):
        pass



def main():
    
    # get command line parameters
    parser = ArgumentParser()

    # arguments
    parser = ArgumentParser(description='versatile curricula generator')

    parser.add_argument('-m','--meta',action="extend", nargs="+", type=str,help="course description(s) as YAML file(s)")
    # parser.add_argument('-l','--lang',help="Language to parse from meta file (use de or en)",default='de')
    # parser.add_argument('-c','--create',action='store_true',help="Create a meta file from description")
    # parser.add_argument('-n','--newpage',action='store_true',help="Create a pagebreak after each table")
    # parser.add_argument('-t','--title',action='store_true',help="Create a title ahead of each table")

    parser.add_argument('-s','--schema',help="using provided schema")
    
    
    # get arguments
    args = parser.parse_args()

    # only run debug
    if args.schema:
        
        cb = CourseBuilder()

        with open(args.schema) as f:
            cb.set_schema(yaml.load(f,Loader=yaml.Loader))

        for m in args.meta:
            with open(m) as fm:
                cb.process(yaml.load(fm,Loader=yaml.Loader))
    else:
        parser.print_help()

if __name__ == '__main__':
    main()
